﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TaskManager.Core;
using TaskManager.DB;
using System.Reflection;

namespace TaskManager.UI
{
    public partial class MainForm : Form
    {
        private string AssemblyTitle
        {
            get
            {
                object[] attributes = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyTitleAttribute), false);
                if (attributes.Length > 0)
                {
                    AssemblyTitleAttribute titleAttribute = (AssemblyTitleAttribute)attributes[0];
                    if (titleAttribute.Title != "")
                        return titleAttribute.Title;
                }
                return System.IO.Path.GetFileNameWithoutExtension(Assembly.GetExecutingAssembly().CodeBase);
            }
        }

        private void InitGroupLists()
        {
            ITaskListsReader reader = XmlTaskListsReaderFactory.Create();
            List<TaskList> taskLists = null;
            try
            {
                taskLists = reader.Read();
            }
            catch (Exception) { }

            if (taskLists != null)
            {
                foreach (TaskList taskList in taskLists)
                {
                    TaskListBox listBox = new TaskListBox(this, cntxtMenu);
                    listBox.Tag = taskList;
                    foreach (Task task in taskList)
                    {
                        listBox.Items.Add(task);
                    }
                    grpTasks.Controls.Add(listBox);
                }
            }

            TaskListBox emptyListBox = new TaskListBox(this, cntxtMenu);
            grpTasks.Controls.Add(emptyListBox);

            ResizeGroupLists();
        }

        private void ResizeGroupLists()
        {
            const int space = 10;
            const int top = 20;
            int n = grpTasks.Controls.Count;
            int width = (int)(((double)grpTasks.Width - (n + 1) * space) / n + 0.5);
            int height = grpTasks.Height - space - top;
            for (int i = 0; i < n; i++)
            {
                grpTasks.Controls[i].Width = width;
                grpTasks.Controls[i].Height = height;
                grpTasks.Controls[i].Top = top;
                grpTasks.Controls[i].Left = (i + 1) * space + i * width;
            }
        }

        private void SaveTaskLists()
        {
            List<TaskList> taskLists = new List<TaskList>();
            for (int i = 0; i < grpTasks.Controls.Count - 1; i++)
            {
                TaskList taskList = grpTasks.Controls[i].Tag as TaskList;
                taskLists.Add(taskList);
            }

            ITaskListsWriter writer = XmlTaskListsWriterFactory.Create();
            writer.Write(taskLists);
        }

        private void Sort()
        {
            for (int i = 0; i < grpTasks.Controls.Count - 1; i++)
            {
                TaskList taskList = grpTasks.Controls[i].Tag as TaskList;
                taskList.Sort();

                TaskListBox listBox = grpTasks.Controls[i] as TaskListBox;
                listBox.Items.Clear();
                foreach (Task task in taskList)
                {
                    listBox.Items.Add(task);
                }
            }
        }

        private TaskListBox GetFocusedListBox()
        {
            TaskListBox focusedListBox = null;
            foreach (Control control in grpTasks.Controls)
            {
                TaskListBox listBox = control as TaskListBox;
                if (listBox.ContainsFocus)
                {
                    focusedListBox = listBox;
                    break;
                }
            }
            return focusedListBox;
        }

        private int GetIndexOnRightClick(TaskListBox focusedListBox)
        {
            if (focusedListBox != null)
            {
                TaskList taskList = focusedListBox.Tag as TaskList;
                if (taskList != null)
                {
                    return focusedListBox.IndexFromPoint(focusedListBox.ContextMenuPoint);
                }
            }
            return ListBox.NoMatches;
        }

        public MainForm()
        {
            InitializeComponent();
        }

        private void 关于AToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AboutForm().ShowDialog();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            InitGroupLists();
        }

        private void grpTasks_SizeChanged(object sender, EventArgs e)
        {
            ResizeGroupLists();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                SaveTaskLists();
            }
            catch (Exception ex)
            {
                if (DialogResult.OK == MessageBox.Show(
                    "由于下列原因，无法保存数据，是否要取消关闭窗体以便重试？" +
                        Environment.NewLine + Environment.NewLine +
                        ex.Message,
                    AssemblyTitle,
                    MessageBoxButtons.OKCancel,
                    MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button1))
                { e.Cancel = true; }
            }
        }

        private class TaskListBox : ListBox
        {
            private MainForm mainForm;
            private Point contextMenuPoint;

            public Point ContextMenuPoint
            {
                get { return contextMenuPoint; }
            }

            public TaskListBox(MainForm mainForm, ContextMenuStrip contextMenuStrip)
                : base()
            {
                this.mainForm = mainForm;
                this.ContextMenuStrip = contextMenuStrip;
                this.contextMenuPoint = new Point(0, 0);
                this.MouseDown += new MouseEventHandler(TaskListBox_MouseDown);
            }

            void TaskListBox_MouseDown(object sender, MouseEventArgs e)
            {
                if (e.Button == MouseButtons.Right)
                    contextMenuPoint = e.Location;
                this.Focus();
                int index = this.IndexFromPoint(e.Location);
                if (index != ListBox.NoMatches)
                    this.SelectedIndex = index;
            }
        }

        private void 添加AToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TaskForm taskForm = new TaskForm();
            if (taskForm.ShowDialog() == DialogResult.OK)
            {
                TaskListBox focusedListBox = GetFocusedListBox();
                if (focusedListBox != null)
                {
                    TaskList taskList = focusedListBox.Tag as TaskList;
                    if (taskList == null)
                    {
                        TaskListBox emptyListBox = new TaskListBox(this, cntxtMenu);
                        grpTasks.Controls.Add(emptyListBox);
                        taskList = new TaskList();
                        focusedListBox.Tag = taskList;
                    }
                    taskList.Append(taskForm.Task);
                    focusedListBox.Items.Add(taskForm.Task);

                    ResizeGroupLists();
                    Sort();
                }
            }
        }

        private void 删除ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TaskListBox focusedListBox = GetFocusedListBox();
            TaskList taskList = focusedListBox.Tag as TaskList;
            int index = GetIndexOnRightClick(focusedListBox);
            if (index == ListBox.NoMatches) return;

            if (MessageBox.Show("删除后无法恢复，请确认删除 " + taskList[index].Name, AssemblyTitle, MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                if (index >= 0 && index < taskList.Count)
                {
                    taskList.Delete(index);
                    focusedListBox.Items.RemoveAt(index);

                    if (focusedListBox.Items.Count == 0)
                    {
                        grpTasks.Controls.Remove(focusedListBox);
                        ResizeGroupLists();
                    }

                    Sort();
                }
            }
        }

        private void 修改MToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TaskListBox focusedListBox = GetFocusedListBox();
            TaskList taskList = focusedListBox.Tag as TaskList;
            int index = GetIndexOnRightClick(focusedListBox);
            if (index != ListBox.NoMatches)
            {
                Task task = GetFocusedListBox().Items[index] as Task;
                TaskForm taskForm = new TaskForm(task);
                if (taskForm.ShowDialog() == DialogResult.OK) Sort();
            }
        }
    }
}
